==> ocre_v0_031.tgz: ocre-0.031/ocre/src/xpm.c <==

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "base.h"
#include "glifos.h"
#include "imag.h"
#include "inclfont.h"

void parametrosDeXpmd (char **xpm_dato, int *dtx, int *dty, int *dncol, int *dncarc)
{
  sscanf (xpm_dato[0],"%d %d %d %d", dtx, dty, dncol, dncarc); 
}

int ncolDeXpmd (char **xpm_dato)
{
  int tx, ty, ncol, ncarc;
  parametrosDeXpmd (xpm_dato, &tx, &ty, &ncol, &ncarc);
  return ncol;
}

void lineaH (char ** xpm_dato, int x1, int y, int x2, char color)
{
  int x;
  int tx, ty, ncol, ncarc;
  parametrosDeXpmd (xpm_dato, &tx, &ty, &ncol, &ncarc);
  if ((y >=0) && (y < ty))
    for (x=x1; x<=x2; x++)
      xpm_dato[y+ncol+1][x] = color ;
}

void lineaV (char ** xpm_dato, int x, int y1, int y2, char color)
{
  int y;
  int ncol;
  ncol = ncolDeXpmd (xpm_dato);
  for (y=y1; y<=y2; y++)
    xpm_dato[y+ncol+1][x] = color ;
}

void lineaIncV (char ** xpm_dato, int x1, int incl, int y1, int y2, char color)
{
  int x, y;
  int ncol;
  ncol = ncolDeXpmd (xpm_dato);
  for (y=y1; y<=y2; y++){
    x = x1 + inclf_desvx(incl, y-y1);
/*
    if ((x2-x1)*(y2-y1) > 0)
      x = x1 + ((x2 - x1)*(y -y1) + (y2 - y1)/2) / (y2 -y1);
    else
      x = x1 + ((x2 - x1)*(y -y1) - (y2 - y1)/2) / (y2 -y1);
*/
    xpm_dato[y+ncol+1][x] = color ;
    }
}

void encuadre (char ** xpm_dato, int x1, int y1, int x2, int y2, int escala, char color)
{
  int xe1, xe2, ye1, ye2;
  int tx, ty, ncol, ncarc;
  parametrosDeXpmd (xpm_dato, &tx, &ty, &ncol, &ncarc);
  xe1 = max(0,    x1 * escala - 1);
  xe2 = min(tx-1, x2 * escala + 1);
  ye1 = max(0,    y1 * escala - 1);
  ye2 = min(ty-1, y2 * escala + 1);
  lineaH (xpm_dato, xe1, ye1, xe2, color);
  lineaH (xpm_dato, xe1, ye2, xe2, color);
  lineaV (xpm_dato, xe1, ye1, ye2, color);
  lineaV (xpm_dato, xe2, ye1, ye2, color);
}

void encuadreInc (char ** xpm_dato, int x1, int y1, int x2, int y2, int incl, int escala, char color)
{
  int xe1, xe2, xe1b, xe2b, ye1, ye2;
  int tx, ty, ncol, ncarc;
  int dxs;
  dxs = inclf_desvx(incl, y2-y1);
  parametrosDeXpmd (xpm_dato, &tx, &ty, &ncol, &ncarc);
  xe1 = x1 * escala - 1;
  xe2 = x2 * escala + 1;
  xe1b = xe1 + dxs * escala;
  xe2b = xe2 + dxs * escala;
  /* en ciertos casos, linea por dentro y no por fuera */
  if (dxs > 0) {
    if ((x2+dxs) >= tx) {
      xe2 -- ;
      xe2b -- ;
      }
    }
  else /* dxs < 0 */ {
    if ((x1+dxs) <= 0) {
      xe1 ++ ;
      xe1b ++ ;
      }
    }
  ye1 = max(0,    y1 * escala - 1);
  ye2 = min(ty-1, y2 * escala + 1);
  lineaH    (xpm_dato, xe1,  ye1, xe2,  color);
  lineaH    (xpm_dato, xe1b, ye2, xe2b, color);
  lineaIncV (xpm_dato, xe1, incl, ye1, ye2, color);
  lineaIncV (xpm_dato, xe2, incl, ye1, ye2, color);
}

void marcaLineasBase (char ** xpm_dato,
	         int x0, int x1, int x2, int x3,
		 int y0, int y1, int escala, char color)
{
  int xe0, xe1, xe2, xe3, ye0, ye1;
  int delta = 3;
  xe0 = x0 * escala;
  xe1 = (x1 - delta) * escala;
  xe2 = (x2 + delta) * escala;
  xe3 = x3 * escala;
  ye0 = max(y0 * escala - 1, 0);  /* no salir fuera */
  ye1 = y1 * escala + 1;
  if ( (xe1 - xe0) >= 3 ) {
    lineaH   (xpm_dato,  xe0, ye0, xe1, color);
    lineaH   (xpm_dato,  xe0, ye1, xe1, color);
    }
  if ( (xe3 - xe2) >= 3 ) {
    lineaH   (xpm_dato,  xe2, ye0, xe3, color);
    lineaH   (xpm_dato,  xe2, ye1, xe3, color);
    }
}

void marcaLineasBaseInc (char ** xpm_dato,
	         int x0, int x1, int x2, int x3,
		 int y0, int y1, int dx, int escala, char color)
{
  int xe0, xe1, xe2, xe3, ye0, ye1;
  int xe1b, xe2b;
  int delta = 3;
  xe0 = x0 * escala;
  xe1 = (x1 - delta) * escala;
  xe1b = xe1 + dx * escala ;
  xe2 = (x2 + delta) * escala;
  xe2b = xe2 + dx * escala ;
  xe3 = x3 * escala;
  ye0 = max(y0 * escala - 1, 0);  /* no salir fuera */
  ye1 = y1 * escala + 1;
  if ( (xe1 - xe0) >= 3 )
    lineaH   (xpm_dato,  xe0, ye0, xe1, color);
  if ( (xe1b - xe0) >= 3 )
    lineaH   (xpm_dato,  xe0, ye1, xe1b, color);
  if ( (xe3 - xe2) >= 3 )
    lineaH   (xpm_dato,  xe2, ye0, xe3, color);
  if ( (xe3 - xe2b) >= 3 )
    lineaH   (xpm_dato,  xe2b, ye1, xe3, color);
}

void liberaXpmDeVentana (char ** xpm_dato)
{
  int i;
  int tx, ty, ncol, ncarc;
  parametrosDeXpmd (xpm_dato, &tx, &ty, &ncol, &ncarc);
  for (i=0; i<(1+ncol+ty); i++)
    free( xpm_dato[ i]);
  free( xpm_dato);
}

void coloresEnXpm (char ** xpm_dato)
{
  int ngris, ncolv;
  ncolv = 4;       /* hay cuatro colores mas: Rojo, Naranja, Verde y aZul */
  ngris = 32;
  {
  int g;
  char g1;
  int g2;
  for (g=0; g < ngris; g++){
    xpm_dato[g+1] = (char *) malloc (12 * sizeof(char));
    if (g <= 9) {
      g1 = g + '0';
      }
    else
     {
     g1 = (g - 10)+ 'a';
     }
    g2 = (g * 255) / 31;
    sprintf (xpm_dato[g+1], "%c c #%02x%02x%02x", g1, g2, g2, g2);
    // sprintf (xpm_dato[g+1], "%c c #%02x%02x%02x\000", g1, g2, g2, g2);
    // sprintf (xpm_dato[g+1], "%c c #%02x%02x%02x%c", g1, g2, g2, g2, '\000');
                                               /* un \0 al final */
    }
  }
  /**/
  xpm_dato[ngris+1] = (char *)strdup("R  c #F00") ;
  xpm_dato[ngris+2] = (char *)strdup("N  c #FA5") ;
  xpm_dato[ngris+3] = (char *)strdup("V  c #0F0") ;
  xpm_dato[ngris+4] = (char *)strdup("Z  c #00F") ;
}

void ponUnPixelVentanaEnXpm (char **xpm_dato, int gp, int x, int y, int ncol_mas1)
{
  int g;
  g = gp / 8;
  if (g <= 9)
    xpm_dato [ncol_mas1+y][x] = '0' + g;
  else
    xpm_dato [ncol_mas1+y][x] = 'a' + (g-10);
}

char **
xpmNuevo (int tx, int ty, int ncol)
{
  char ** xpm_dato;
  xpm_dato = (char **) malloc ((ncol + 1+ ty) * sizeof(char *));
  /**/
  xpm_dato[ 0] = (char *) malloc (16 * sizeof(char));
  sprintf (xpm_dato[ 0], "%4d %4d %3d %1d", tx, ty, ncol, 1);
  // sprintf (xpm_dato[ 0], "%4d %4d %3d %1d\000", tx, ty, ncol, 1);
  // sprintf (xpm_dato[ 0], "%4d %4d %3d %1d%c", tx, ty, ncol, 1, '\000');
                                               /* un \0 al final */
  return xpm_dato;
}

void pixelsEnXpm (char **xpm_dato, int tx, int ty, int ncol, char **pix, int escala)
{
  int ncol_mas1;
  int x, y;
  ncol_mas1 = ncol + 1;
  for (y=0; y<ty; y++){
    xpm_dato [ncol_mas1+y] = (char *) malloc ( tx * sizeof(char));
    for (x=0; x<tx; x++)
      ponUnPixelVentanaEnXpm (xpm_dato,
		             (pix[y/escala][x/escala] & 0x0FF),
			     x, y, ncol_mas1) ;
    }
}

char **
xpmDeVentanaRec (tvent vent, int tdx, int tdy)
{
  char ** xpm_dato;
  int tx, ty;
  int escala;
  int ngris, ncol, ncolv;
  ncolv = 4;       /* hay cuatro colores mas: Rojo, Naranja, Verde y aZul */
  ngris = 32;
  ncol = ngris + ncolv;
  /**/
  escala = 2;
  tx  = max( min(tdx,vent->imaBase->ven0->tx * escala), vent->tx * escala ) ;
  ty  = max( min(tdy,vent->imaBase->ven0->ty * escala), vent->ty * escala ) ;
  xpm_dato = xpmNuevo (tx, ty, ncol) ;
  coloresEnXpm (xpm_dato);
  {
  tvent vent2;
  int base, alt;
  int base2, alturax2;
  ventanaAlrededor (&vent2, vent, tx, ty, escala) ;
  pixelsEnXpm (xpm_dato, tx, ty, ncol, vent2->pix, escala);
  encuadre (xpm_dato,
            vent->dx - vent2->dx,
            vent->dy - vent2->dy,
	    vent->dx - vent2->dx + vent->tx -1,
	    vent->dy - vent2->dy + vent->ty -1,
	    escala, 'R');
  alt  = vent->dy - vent2->dy ;
  base = vent->dy - vent2->dy + vent->ty -1 ;
  baseyalturaxDeGlifo (&base2, &alturax2, GLIfoDeVen(vent));
  base2    -= vent2->dy ;
  alturax2 -= vent2->dy ;
//  printf ("alt: %4d, base: %4d, alturax2: %4d, base2: %4d, vent->dy: %4d\n",
//           alt,      base,      alturax2,      base2,      vent->dy);
  marcaLineasBase (xpm_dato,
              1,
              vent->dx - vent2->dx - 2,
              vent->dx - vent2->dx + vent->tx -1 + 2,
              (tx/escala) - 2,
              alt, base,
              escala, 'Z');
  marcaLineasBase (xpm_dato,
              1,
              vent->dx - vent2->dx - 2,
              vent->dx - vent2->dx + vent->tx -1 + 2,
              (tx/escala) - 2,
              alturax2, base2,
              escala, 'V');
  destruyeVentana (vent2);
  }
  return xpm_dato ;
}

char **
xpmDeVentanaInc (tvent vent, int tdx, int tdy)
{
  char ** xpm_dato;
  int tx, ty;
  int escala;
  int ngris, ncol, ncolv;
  ncolv = 4;       /* hay cuatro colores mas: Rojo, Naranja, Verde y aZul */
  ngris = 32;
  ncol = ngris + ncolv;
  /**/
  escala = 2;
  tx  = min(tdx,vent->imaBase->ven0->tx * escala) ;
  ty  = min(tdy,vent->imaBase->ven0->ty * escala) ;
  xpm_dato = xpmNuevo (tx, ty, ncol) ;
  coloresEnXpm (xpm_dato);
  {
  tvent vent2;
  int vent_dx, vent_tx;
  int dxs;
  ventanaAlrededor (&vent2, vent, tx, ty, escala) ;
  pixelsEnXpm (xpm_dato, tx, ty, ncol, vent2->pix, escala);
  dxs = inclf_desvx(vent->incl, vent->ty - 1);
  if (vent->incl > 0) {
    vent_dx = vent->dx ;
    vent_tx = vent->tx - dxs;
    }
  else  /* vent->incl < 0 */ {
    vent_dx = vent->dx - dxs;
    vent_tx = vent->tx + dxs;
    }
  encuadreInc (xpm_dato,
               vent_dx  - vent2->dx,
               vent->dy - vent2->dy,
               vent_dx  - vent2->dx + vent_tx -1,
               vent->dy - vent2->dy + vent->ty -1,
               vent->incl,
               escala, 'R');
  marcaLineasBaseInc (xpm_dato,
               1,
               vent_dx  - vent2->dx - 2,
               vent_dx  - vent2->dx + vent_tx -1 + 2,
               (tx/escala) - 2,
               vent->dy - vent2->dy ,
               vent->dy - vent2->dy + vent->ty -1,
               dxs,
               escala, 'Z');
  destruyeVentana (vent2);
  }
  return xpm_dato ;
}

char **
xpmDeVentana (tvent vent, int tdx, int tdy)
{
  if (vent->incl == 0)
    return xpmDeVentanaRec (vent, tdx, tdy);
  else  /* vent->incl != 0 */
    return xpmDeVentanaInc (vent, tdx, tdy);
}

==> ocre_v0_042.tgz: ocre-0.042/ocre/src/xpm.c <==

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "base.h"
#include "glifos.h"
#include "imag.h"
// #include "inclfont.h"
#include "inclina0.h"

void parametrosDeXpmd (char **xpm_dato, int *dtx, int *dty, int *dncol, int *dncarc)
{
  sscanf (xpm_dato[0],"%d %d %d %d", dtx, dty, dncol, dncarc); 
}

int ncolDeXpmd (char **xpm_dato)
{
  int tx, ty, ncol, ncarc;
  parametrosDeXpmd (xpm_dato, &tx, &ty, &ncol, &ncarc);
  return ncol;
}

void lineaH (char ** xpm_dato, int x1, int y, int x2, char color)
{
  int x;
  int tx, ty, ncol, ncarc;
  parametrosDeXpmd (xpm_dato, &tx, &ty, &ncol, &ncarc);
  if ((y >=0) && (y < ty))
    for (x=x1; x<=x2; x++)
      xpm_dato[y+ncol+1][x] = color ;
}

void lineaV (char ** xpm_dato, int x, int y1, int y2, char color)
{
  int y;
  int ncol;
  ncol = ncolDeXpmd (xpm_dato);
  for (y=y1; y<=y2; y++)
    xpm_dato[y+ncol+1][x] = color ;
}

void lineaIncV (char ** xpm_dato, int x1, int incl, int y1, int y2, char color)
{
  int x, y;
  int ncol;
  ncol = ncolDeXpmd (xpm_dato);
  for (y=y1; y<=y2; y++){
    x = x1 + inclf_desvx(incl, y-y1);
/*
    if ((x2-x1)*(y2-y1) > 0)
      x = x1 + ((x2 - x1)*(y -y1) + (y2 - y1)/2) / (y2 -y1);
    else
      x = x1 + ((x2 - x1)*(y -y1) - (y2 - y1)/2) / (y2 -y1);
*/
    xpm_dato[y+ncol+1][x] = color ;
    }
}

void encuadre (char ** xpm_dato, int x1, int y1, int x2, int y2, int escala, char color)
{
  int xe1, xe2, ye1, ye2;
  int tx, ty, ncol, ncarc;
  parametrosDeXpmd (xpm_dato, &tx, &ty, &ncol, &ncarc);
  xe1 = max(0,    x1 * escala - 1);
  xe2 = min(tx-1, x2 * escala + 1);
  ye1 = max(0,    y1 * escala - 1);
  ye2 = min(ty-1, y2 * escala + 1);
  lineaH (xpm_dato, xe1, ye1, xe2, color);
  lineaH (xpm_dato, xe1, ye2, xe2, color);
  lineaV (xpm_dato, xe1, ye1, ye2, color);
  lineaV (xpm_dato, xe2, ye1, ye2, color);
}

void encuadreInc (char ** xpm_dato, int x1, int y1, int x2, int y2, int incl, int escala, char color)
{
  int xe1, xe2, xe1b, xe2b, ye1, ye2;
  int tx, ty, ncol, ncarc;
  int dxs;
  dxs = inclf_desvx(incl, y2-y1);
  parametrosDeXpmd (xpm_dato, &tx, &ty, &ncol, &ncarc);
  xe1 = x1 * escala - 1;
  xe2 = x2 * escala + 1;
  xe1b = xe1 + dxs * escala;
  xe2b = xe2 + dxs * escala;
  /* en ciertos casos, linea por dentro y no por fuera */
  if (dxs > 0) {
    if ((x2+dxs) >= tx) {
      xe2 -- ;
      xe2b -- ;
      }
    }
  else /* dxs < 0 */ {
    if ((x1+dxs) <= 0) {
      xe1 ++ ;
      xe1b ++ ;
      }
    }
  ye1 = max(0,    y1 * escala - 1);
  ye2 = min(ty-1, y2 * escala + 1);
  lineaH    (xpm_dato, xe1,  ye1, xe2,  color);
  lineaH    (xpm_dato, xe1b, ye2, xe2b, color);
  lineaIncV (xpm_dato, xe1, incl, ye1, ye2, color);
  lineaIncV (xpm_dato, xe2, incl, ye1, ye2, color);
}

void marcaLineasBase (char ** xpm_dato,
	         int x0, int x1, int x2, int x3,
		 int y0, int y1, int escala, char color)
{
  int xe0, xe1, xe2, xe3, ye0, ye1;
  int delta = 3;
  xe0 = x0 * escala;
  xe1 = (x1 - delta) * escala;
  xe2 = (x2 + delta) * escala;
  xe3 = x3 * escala;
  ye0 = max(y0 * escala - 1, 0);  /* no salir fuera */
  ye1 = y1 * escala + 1;
  if ( (xe1 - xe0) >= 3 ) {
    lineaH   (xpm_dato,  xe0, ye0, xe1, color);
    lineaH   (xpm_dato,  xe0, ye1, xe1, color);
    }
  if ( (xe3 - xe2) >= 3 ) {
    lineaH   (xpm_dato,  xe2, ye0, xe3, color);
    lineaH   (xpm_dato,  xe2, ye1, xe3, color);
    }
}

void marcaLineasBaseInc (char ** xpm_dato,
	         int x0, int x1, int x2, int x3,
		 int y0, int y1, int dx, int escala, char color)
{
  int xe0, xe1, xe2, xe3, ye0, ye1;
  int xe1b, xe2b;
  int delta = 3;
  xe0 = x0 * escala;
  xe1 = (x1 - delta) * escala;
  xe1b = xe1 + dx * escala ;
  xe2 = (x2 + delta) * escala;
  xe2b = xe2 + dx * escala ;
  xe3 = x3 * escala;
  ye0 = max(y0 * escala - 1, 0);  /* no salir fuera */
  ye1 = y1 * escala + 1;
  if ( (xe1 - xe0) >= 3 )
    lineaH   (xpm_dato,  xe0, ye0, xe1, color);
  if ( (xe1b - xe0) >= 3 )
    lineaH   (xpm_dato,  xe0, ye1, xe1b, color);
  if ( (xe3 - xe2) >= 3 )
    lineaH   (xpm_dato,  xe2, ye0, xe3, color);
  if ( (xe3 - xe2b) >= 3 )
    lineaH   (xpm_dato,  xe2b, ye1, xe3, color);
}

void liberaXpmDeVentana (char ** xpm_dato)
{
  int i;
  int tx, ty, ncol, ncarc;
  parametrosDeXpmd (xpm_dato, &tx, &ty, &ncol, &ncarc);
  for (i=0; i<(1+ncol+ty); i++)
    free( xpm_dato[ i]);
  free( xpm_dato);
}

void coloresEnXpm (char ** xpm_dato)
{
  int ngris, ncolv;
  ncolv = 4;       /* hay cuatro colores mas: Rojo, Naranja, Verde y aZul */
  ngris = 32;
  {
  int g;
  char g1;
  int g2;
  for (g=0; g < ngris; g++){
    xpm_dato[g+1] = (char *) malloc (12 * sizeof(char));
    if (g <= 9) {
      g1 = g + '0';
      }
    else
     {
     g1 = (g - 10)+ 'a';
     }
    g2 = (g * 255) / 31;
    sprintf (xpm_dato[g+1], "%c c #%02x%02x%02x", g1, g2, g2, g2);
    // sprintf (xpm_dato[g+1], "%c c #%02x%02x%02x\000", g1, g2, g2, g2);
    // sprintf (xpm_dato[g+1], "%c c #%02x%02x%02x%c", g1, g2, g2, g2, '\000');
                                               /* un \0 al final */
    }
  }
  /**/
  xpm_dato[ngris+1] = (char *)strdup("R  c #F00") ;
  xpm_dato[ngris+2] = (char *)strdup("N  c #FA5") ;
  xpm_dato[ngris+3] = (char *)strdup("V  c #0F0") ;
  xpm_dato[ngris+4] = (char *)strdup("Z  c #00F") ;
}

void ponUnPixelVentanaEnXpm (char **xpm_dato, int gp, int x, int y, int ncol_mas1)
{
  int g;
  g = gp / 8;
  if (g <= 9)
    xpm_dato [ncol_mas1+y][x] = '0' + g;
  else
    xpm_dato [ncol_mas1+y][x] = 'a' + (g-10);
}

char **
xpmNuevo (int tx, int ty, int ncol)
{
  char ** xpm_dato;
  xpm_dato = (char **) malloc ((ncol + 1+ ty) * sizeof(char *));
  /**/
  xpm_dato[ 0] = (char *) malloc (16 * sizeof(char));
  sprintf (xpm_dato[ 0], "%4d %4d %3d %1d", tx, ty, ncol, 1);
  // sprintf (xpm_dato[ 0], "%4d %4d %3d %1d\000", tx, ty, ncol, 1);
  // sprintf (xpm_dato[ 0], "%4d %4d %3d %1d%c", tx, ty, ncol, 1, '\000');
                                               /* un \0 al final */
  return xpm_dato;
}

void pixelsEnXpm (char **xpm_dato, int tx, int ty, int ncol, char **pix, int escala)
{
  int ncol_mas1;
  int x, y;
  ncol_mas1 = ncol + 1;
  for (y=0; y<ty; y++){
    xpm_dato [ncol_mas1+y] = (char *) malloc ( tx * sizeof(char));
    for (x=0; x<tx; x++)
      ponUnPixelVentanaEnXpm (xpm_dato,
		             (pix[y/escala][x/escala] & 0x0FF),
			     x, y, ncol_mas1) ;
    }
}

char **
xpmDeVentanaRec (tvent vent, int tdx, int tdy)
{
  char ** xpm_dato;
  int tx, ty;
  int escala;
  int ngris, ncol, ncolv;
  ncolv = 4;       /* hay cuatro colores mas: Rojo, Naranja, Verde y aZul */
  ngris = 32;
  ncol = ngris + ncolv;
  /**/
  escala = 2;
  tx  = max( min(tdx,vent->imaBase->ven0->tx * escala), vent->tx * escala ) ;
  ty  = max( min(tdy,vent->imaBase->ven0->ty * escala), vent->ty * escala ) ;
  xpm_dato = xpmNuevo (tx, ty, ncol) ;
  coloresEnXpm (xpm_dato);
  {
  tvent vent2;
  int base, alt;
  int base2, alturax2;
  ventanaAlrededor (&vent2, vent, tx, ty, escala) ;
  pixelsEnXpm (xpm_dato, tx, ty, ncol, vent2->pix, escala);
  encuadre (xpm_dato,
            vent->dx - vent2->dx,
            vent->dy - vent2->dy,
	    vent->dx - vent2->dx + vent->tx -1,
	    vent->dy - vent2->dy + vent->ty -1,
	    escala, 'R');
  alt  = vent->dy - vent2->dy ;
  base = vent->dy - vent2->dy + vent->ty -1 ;
  baseyalturaxDeGlifo (&base2, &alturax2, GLIfoDeVen(vent));
  base2    -= vent2->dy ;
  alturax2 -= vent2->dy ;
//  printf ("alt: %4d, base: %4d, alturax2: %4d, base2: %4d, vent->dy: %4d\n",
//           alt,      base,      alturax2,      base2,      vent->dy);
  marcaLineasBase (xpm_dato,
              1,
              vent->dx - vent2->dx - 2,
              vent->dx - vent2->dx + vent->tx -1 + 2,
              (tx/escala) - 2,
              alt, base,
              escala, 'Z');
  marcaLineasBase (xpm_dato,
              1,
              vent->dx - vent2->dx - 2,
              vent->dx - vent2->dx + vent->tx -1 + 2,
              (tx/escala) - 2,
              alturax2, base2,
              escala, 'V');
  destruyeVentana (vent2);
  }
  return xpm_dato ;
}

char **
xpmDeVentanaInc (tvent vent, int tdx, int tdy)
{
  char ** xpm_dato;
  int tx, ty;
  int escala;
  int ngris, ncol, ncolv;
  ncolv = 4;       /* hay cuatro colores mas: Rojo, Naranja, Verde y aZul */
  ngris = 32;
  ncol = ngris + ncolv;
  /**/
  escala = 2;
  tx  = min(tdx,vent->imaBase->ven0->tx * escala) ;
  ty  = min(tdy,vent->imaBase->ven0->ty * escala) ;
  xpm_dato = xpmNuevo (tx, ty, ncol) ;
  coloresEnXpm (xpm_dato);
  {
  tvent vent2;
  int vent_dx, vent_tx;
  int dxs;
  ventanaAlrededor (&vent2, vent, tx, ty, escala) ;
  pixelsEnXpm (xpm_dato, tx, ty, ncol, vent2->pix, escala);
  dxs = inclf_desvx(vent->incl, vent->ty - 1);
  if (vent->incl > 0) {
    vent_dx = vent->dx ;
    vent_tx = vent->tx - dxs;
    }
  else  /* vent->incl < 0 */ {
    vent_dx = vent->dx - dxs;
    vent_tx = vent->tx + dxs;
    }
  encuadreInc (xpm_dato,
               vent_dx  - vent2->dx,
               vent->dy - vent2->dy,
               vent_dx  - vent2->dx + vent_tx -1,
               vent->dy - vent2->dy + vent->ty -1,
               vent->incl,
               escala, 'R');
  marcaLineasBaseInc (xpm_dato,
               1,
               vent_dx  - vent2->dx - 2,
               vent_dx  - vent2->dx + vent_tx -1 + 2,
               (tx/escala) - 2,
               vent->dy - vent2->dy ,
               vent->dy - vent2->dy + vent->ty -1,
               dxs,
               escala, 'Z');
  destruyeVentana (vent2);
  }
  return xpm_dato ;
}

char **
xpmDeVentana (tvent vent, int tdx, int tdy)
{
  if (vent->incl == 0)
    return xpmDeVentanaRec (vent, tdx, tdy);
  else  /* vent->incl != 0 */
    return xpmDeVentanaInc (vent, tdx, tdy);
}
